# -*- python -*-
# $RCSfile: materialplugin.py,v $
# $Revision: 1.14 $
# $Author: langer $
# $Date: 2010-04-28 14:55:58 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@nist.gov. 

## Define a MicrostructurePlugIn to catch "material changed"
## switchboard signals and tell the relevant subproblems of the event.
from ooflib.SWIG.common import config
from ooflib.SWIG.common import switchboard
from ooflib.SWIG.engine import material
from ooflib.common import debug
from ooflib.common import microstructure
from ooflib.engine import conjugate
from ooflib.engine import materialmanager
from ooflib.engine import meshstatus
from ooflib.engine import subproblemcontext


class MaterialMSPlugIn(microstructure.MicrostructurePlugIn):
    def __init__(self, ms):
        microstructure.MicrostructurePlugIn.__init__(self, ms)
        self.sbcallback = switchboard.requestCallback("material changed",
                                                      self.matChanged)
    def destroy(self):
        switchboard.removeCallback(self.sbcallback)
        microstructure.MicrostructurePlugIn.destroy(self)
    def matChanged(self, materialname):
        # The given material has changed.  Look through the materials
        # in our Microstructure to see if we care.
        matl = materialmanager.getMaterial(materialname)
        #Interface branch
        ##TODO: Should we look at the material associated with the
        ##edgements instead?
        if config.dimension() == 2:
            interfacemsplugin=self.microstructure.getPlugIn("Interfaces")
            interfacemats=interfacemsplugin.getInterfaceMaterials()
        elif config.dimension() == 3:
            interfacemats = []

        if matl in material.getMaterials(self.microstructure) or \
               materialname in interfacemats:
            # We *do* care. Really.
            # Tell all the SubProblems belonging to the microstructure that
            # the material has changed.
            ## TODO: The following code should do what the comment
            ## says, and just tell the SubProblem that the material
            ## has changed.  It's the SubProblem's job to decide what
            ## to do with that information, such as check solvability,
            ## etc.
            msname = self.microstructure.name()
            subppaths = subproblemcontext.subproblems.keys(base=msname)
            for subppath in subppaths:
                subpctxt = subproblemcontext.subproblems[[msname]+subppath]
                subpctxt.changed("Material properties changed.")
                solvability = subpctxt.checkSolvability()
                if solvability:
                    subpctxt.getParent().setStatus(
                        meshstatus.Unsolvable(solvability))
                conjugate.check_symmetry([msname]+subppath)
                

microstructure.registerMicrostructurePlugIn(MaterialMSPlugIn, "Material")
