# -*- python -*-
# $RCSfile: steppererror.py,v $
# $Revision: 1.3 $
# $Author: langer $
# $Date: 2009-03-30 20:55:45 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from ooflib.common import registeredclass

import itertools
import math

raise RunTimeError
OBSOLETE

# Error scaling and tolerance for TimeSteppers

class StepperError(registeredclass.Registeredclasses, object):
    registry = []
    # Subclasses must provide the following two methods:
    def scaling(self, vals, derivs, stepsize):
        # Returns a vector (or generator) of values that will divide
        # the errors before they're compared to the tolerance.
        pass
    def __call__(self, vals, derivs, errors, stepsize):
        # Returns the ratio of the maximum error to the tolerance.  If
        # the ratio is less than one, the error condition is
        # satisfied.
        pass


    def maxerror(self, vals, derivs, errors, stepsize):
        # Returns the ratio of the actual error to the desired error.
        scales = self.scaling(vals, derivs, stepsize)
        # This should be done in C++. 
        return max([math.fabs(v/s) for v,s in itertools.izip(vals, scales)])

#########################

class SingleErrorScaling(StepperError):
    def __init__(self, tolerance):
        self.tolerance = tolerance
    def __call__(self, vals, derivs, errors, stepsize):
        return self.maxerror(vals, derivs, errors, stepsize)/self.tolerance

#########################

class AbsoluteError(SingleErrorScaling):
    def scaling(self, vals, derivs, stepsize):
        return itertools.repeat(1.0)

registeredclass.Registration(
    'Absolute',
    StepperError,
    RelativeError,
    ordering=0,
    params=[
        parameter.FloatParameter('tolerance', 1.e-6,
                                 tip='Allowed error in each variable')], 
    tip='The error in each variable must be less than the given tolerance.')

#########################

class RelativeError(SingleErrorScaling):
    def scaling(self, vals, derivs):
        return vals

registeredclass.Registration(
    'Relative',
    StepperError,
    RelativeError,
    ordering=1,
    params=[parameter.FloatParameter('tolerance', 1.e-6,
                                     tip='Allowed relative error')], 
    tip='The error in each variable must be less than the given tolerance times the value of the variable.')

#########################

class ModifiedRelativeError(SingleErrorScaling):
    def scaling(self, vals, derivs, stepsize):
        for v, d in itertools.izip(vals, derivs):
            yield math.fabs(v) + math.fabs(stepsize*d)

registeredclass.Registration(
    'ModifiedRelative',
    StepperError,
    ModifiedRelativeError,
    ordering=1.1,
    params=[parameter.FloatParameter('tolerance', 1.e-6,
                                     tip='Allowed error')], 
    tip='Like Relative error, except for variables near 0, when it\'s more like Global error.')

#########################

class GlobalError(SingleErrorScaling):
    def scaling(self, vals, derivs, stepsize):
        for d in derivs:
            yield d*stepsize
        
registeredclass.Registration(
    'Global',
    StepperError,
    GlobalError,
    ordering=1.1,
    params=[parameter.FloatParameter('tolerance', 1.e-6,
                                     tip='Allowed error')], 
    tip='Accumulated error in each variable must be less than the given tolerance.')

#########################

## TODO: Write a ComboError class, which can apply different
## SingleErrorScaling objects to different Fields.
    
