# -*- python -*-
# $RCSfile: regression.py,v $
# $Revision: 1.54 $
# $Author: langer $
# $Date: 2010-11-04 18:10:50 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@nist.gov.



# Top-level regression test file for the OOF application.  Knows about
# all the test suites in this directory, and what order to run them in
# in order to get a proper regression test.

import sys, imp, os, getopt, copy

# The startup sequence for regression.py has to imitate the executable
# oof2 script. That one imports the contents of the math module into
# the main oof namespace, so we have to do it here too.  Not importing
# math here will make some tests fail.
from math import *

## TODO: Use a temp directory for all output files, so that the tests
## can be run simultaneously on two computers with a shared file
## system.

test_module_names = [
    "fundamental_test",
    "microstructure_test",
    "image_test",
    "pixel_test",
    "activearea_test",
    "microstructure_extra_test",
    "matrix_test",
    "skeleton_basic_test",
    "skeleton_select_test",
    "skeleton_bdy_test",
    "skeleton_periodic_test",
    "skeleton_periodic_bdy_test",
    "skeleton_selectionmod_test",
    "skeleton_extra_test",
    "material_property_test",
    "pixel_extra_test",
    "mesh_test",
    "subproblem_test",
    "solver_test",
    "boundary_condition_test",
    "aniso_test",
    "nonlinear_linear_test",
    "nonlinear_floatbc_test",
    "nonconstant_property_test",
    "nonlinear_property_test",
    "nonlinear_plane_flux_test",
    "nonlinear_timedependent_tests",
    "nonlinear_K_timedep_tests",
    "amr_test",
    "output_test",
    "subproblem_test_extra",
    "r3tensorrotationbug",
    # "interface_test"
    ]

def stripdotpy(name):
    if name.endswith(".py"):
        return name[:-3]
    return name

if __name__=="__main__":
    try:
        opts,args = getopt.getopt(sys.argv[1:],"f:a:t:o:",
                                  ["from=", "after=", "to=", "oofargs="])
    except getopt.GetoptError, err:
        print str(err)
        print "Usage: regression.py [--start <starttest> | --after <starttest>] [--end <endtest>] [--oofargs <oofargs>] [tests]"
        print "       Don't use --start or --end if tests are listed explicitly."
        sys.exit(2)

    oofargs = []
    
    startendgiven = False
    startaftergiven = False

    for o,v in opts:
        if o in ("-f", "--from"):
            if startaftergiven:
                print >> sys.stderr, "You can't use both --start and --after."
                sys.exit(1)
            v = stripdotpy(v)
            test_module_names = test_module_names[test_module_names.index(v):]
            startendgiven = True
            startaftergiven = True
        if o in ("-a", "--after"):
            if startaftergiven:
                print >> sys.stderr, "You can't use both --start and --after."
                sys.exit(1)
            v = stripdotpy(v)
            test_module_names = \
                test_module_names[test_module_names.index(v)+1:]
            startendgiven = True
            startaftergiven = True
        elif o in ("-t", "--to"):
            v = stripdotpy(v)
            test_module_names = \
                test_module_names[:test_module_names.index(v)+1]
            startendgiven = True
        elif o in ("-o","--oofargs"):
            oofargs = v.split()

    if startendgiven:
        if args:
            print "You can't explicitly list the tests *and* use --start, --after, or --end."
            sys.exit(1)
    elif args:
        test_module_names = [stripdotpy(a) for a in args]
        

    # Effectively pass these through.
    sys.argv = [sys.argv[0]] + oofargs

    try:
        import oof2
        sys.path.append(os.path.dirname(oof2.__file__))
        from ooflib.common import oof
    except ImportError:
        print "OOF is not correctly installed on this system."
        sys.exit(4)

    sys.argv.extend(["--text", "--quiet", "--seed=17"])
    oof.run(no_interp=1)

    # globals() contains OOF namespace objects that we will be making
    # available to each test script.  If test scripts modify globals
    # (eg, by using utils.OOFdefine or the scriptloader), we don't
    # want those modifications to affect later test scripts.
    # Therefore we create a pristine copy of globals now, and use it
    # instead of globals() later.
    oofglobals = copy.copy(globals())

    for m in test_module_names:
        try:
            exec "import " + m + " as test_module"
        except ImportError:
            print "Unable to load module '%s'." % m
        else:
            print "Running test module %s." % m
            # Make sure all the goodies in the OOF namespace are available.
            test_module.__dict__.update(oofglobals)
            res = test_module.run_tests()
            if res==0: # failure.
                break
    OOF.File.Quit()
